#
#Author: Russell Nelson
#Licnese: GPL
#Date: 03/22/2001
#Program: ipaddr.py
#Version: 0.01
#
name = "IPaddr"
desc = "Set System's IP address"
glade_file = 'ipaddr.glade'

from gtk import *
from GTK import *
from utils import *
import string

def int_to_ip_addr(ip):
    return "%u.%u.%u.%u" % ((ip >>24 & 255), (ip>>16 & 255), (ip>>8 & 255), (ip & 255))

def ip_addr_to_int(ipaddr):
    s = string.split(ipaddr, '.')
    if len(s) != 4:
        return 0
    try:
        return (int(s[0])<<24) + (int(s[1])<<16) + (int(s[2])<<8) + int(s[3])
    except:
        return 0

class applet(Applet):
    manualnames = ['IPADDR', 'NETMASK', 'NETWORK', 'BROADCAST', 'GATEWAY']
    fn = '/etc/pcmcia/network.opts'
    fn = '/tmp/network.opts'

    def on_gateway_changed(self, widget):
        ipaddr = ip_addr_to_int(self.widgets['IPADDR'].get_text())
        netmask = ip_addr_to_int(self.widgets['NETMASK'].get_text())
        gateway = ip_addr_to_int(self.widgets['GATEWAY'].get_text())
        style = widget.get_style().copy()
        if ipaddr & netmask == gateway & netmask:
            self.widgets['status'].set_text("")
            style.fg[STATE_NORMAL] = style.black
        else:
            self.widgets['status'].set_text("Gateway must be on your subnet.")
            style.fg[STATE_NORMAL] = self.get_colormap().alloc('red') 
        widget.set_style(style)

    def on_subnetbits_value_changed(self, adj):
        netmask = ~0 << (32-int(adj.value))
        self.widgets['NETMASK'].set_text(int_to_ip_addr(netmask))
        ipaddr = ip_addr_to_int(self.widgets['IPADDR'].get_text())
        if ipaddr:
            self.widgets['NETWORK'].set_text(int_to_ip_addr(ipaddr & netmask))
            self.widgets['BROADCAST'].set_text(int_to_ip_addr(ipaddr | ~netmask))
        self.on_gateway_changed(self.widgets['GATEWAY'])

    def __init__(self,glade_file):
        import string

        Applet.__init__(self,glade_file)

        self.widgets['subnetbits'].get_adjustment().connect("value_changed", self.on_subnetbits_value_changed)
        self.widgets['GATEWAY'].connect("changed", self.on_gateway_changed)
        style = self.widgets['status'].get_style().copy()
        style.fg[STATE_NORMAL] = self.widgets['status'].get_colormap().alloc('red') 
        self.widgets['status'].set_style(style)

        try:
            inf = open(self.fn)
            while 1:
                line = inf.readline()
                if not line: break
                words = string.split(string.strip(line), '=')
                if len(words) == 2:
                    name = words[0]
                    val = words[1][1:-1]   # strip off quotes
                    if name == "BOOTP" and val == "y":
                        self.widgets['notebook1'].set_page(0)
                    if name == "DHCP" and val == "y":
                        self.widgets['notebook1'].set_page(1)
                    if name == "IPADDR" and val != "":
                        self.widgets['notebook1'].set_page(2)
                    if name in self.manualnames:
                        self.widgets[name].set_text(val)
            inf.close()
        except:
            raise

    def save(self,msgbox=FALSE):
        try:
            inf = open(self.fn)
            outf = open(self.fn+'.new','w')
            while 1:
                line = inf.readline()
                if not line: break
                words = string.split(string.strip(line), '=')
                if len(words) == 2:
                    name = words[0]
                    val = words[1][1:-1]   # strip off quotes
                    if name == "BOOTP":
                        val = "n"
                        if self.widgets['notebook1'].get_current_page() == 0:
                            val = "y"
                    if name == "DHCP":
                        val = "n"
                        if self.widgets['notebook1'].get_current_page() == 1:
                            val = "y"
                    if name == "PUMP":
                        val = "n"
                        if self.widgets['notebook1'].get_current_page() == 1 and self.widgets['PUMP'].get_active():
                            val = "y"
                    if name in self.manualnames:
                        if self.widgets['notebook1'].get_current_page() == 2:
                            val = self.widgets[name].get_text()
                        else:
                            val = ""
                    outf.write('    '+name+'="'+val+'"\n')
                else:
                    outf.write(line)
            inf.close()
            outf.close()
            os.rename(self.fn+'.new', self.fn)
        except:
          raise
        Applet.save(self)

applet_instance = applet(glade_file)
